const { prompt, Toggle } = require('enquirer')
const byuOAuth = require('byu-wabs-oauth')

;(async function () {
  const args = process.argv.slice(2) // there is always at least 2, due to 'node' and 'index'. We only care about anything after that.
  printBold('BYU-OIT Training Token Generator\n')
  if (args.length === 0 && !process.env.stage && !process.env.clientId && !process.env.clientSecret) {
    printBold('For faster usage, run `node index [dev/prod] [your client id] [your client secret]`')
    printBold('Alternatively, see the readme for storing these as env variables\n')
  }
  let stage
  if (args.length > 0) { // default to arguments if provided
    stage = args[0].toLowerCase()
  } else if (process.env.stage) { // then check for env variables
    stage = process.env.stage.toLowerCase()
  } else {
    const tog = new Toggle({
      disabled: 'Sandbox',
      enabled: 'Production',
      message: 'Please select your environment (toggle with arrow keys)'
    })
    stage = await tog.run() ? 'prod' : 'dev'
  }
  if (stage === 'prd' || stage === 'production' || stage === 'stable') { // this logic is to verify the env and cli args
    stage = 'prod'
  }
  let clientId
  if (args.length > 1) {
    clientId = args[1]
  } else if (process.env.clientId) {
    clientId = process.env.clientId
  } else {
    const res = await prompt({
      type: 'input',
      name: 'response',
      message: 'Please enter your Client Id'
    })
    clientId = res.response
  }
  let clientSecret
  if (args.length > 2) {
    clientSecret = args[2]
  } else if (process.env.clientSecret) {
    clientSecret = process.env.clientSecret
  } else {
    const res = await prompt({
      type: 'password',
      name: 'response',
      message: 'Please enter your Client Secret'
    })
    clientSecret = res.response
  }
  let token
  try {
    const host = stage === 'prod' ? 'api.byu.edu' : 'api-sandbox.byu.edu'
    const oauth = await byuOAuth(clientId, clientSecret, { host })
    token = await oauth.getClientGrantToken()
  } catch (e) {
    if (e.message.includes('Unable to get access token')) {
      printBold('Unable to get token. Things to check:')
      printBold('Make sure you have entered in your Client Id and Secret in correctly')
      printBold('Make sure that your stage matches your credentials (e.g. you need Sandbox credentials to generate a token in the Sandbox environment)')
      printBold('If using CLI arguments, make sure you are following the right order (env, client id, client secret)')
      process.exit(-1)
    } else {
      throw e
    }
  }
  printBold(`${stage === 'prod' ? 'PRODUCTION' : 'SANDBOX'} TOKEN:\n`)
  printBold(token.accessToken)
  printBold('\nIt expires in one hour from now (at ' + token.expiresAt + ')')
  process.exit(0)
})()

function printBold(string) { // used to mimic the text style used by 'enquirer'
  console.log('\x1b[1m%s\x1b[0m',string)
}
